unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, Grids, ExtCtrls,
  StdCtrls, Spin, TAGraph, TASources, TASeries, Types, TAChartUtils, TADelaunay;

type

  { TForm1 }

  TForm1 = class(TForm)
    Chart1: TChart;
    Chart1UserDrawnSeries1: TUserDrawnSeries;
    fseAlpha: TFloatSpinEdit;
    Label1: TLabel;
    Panel1: TPanel;
    Panel2: TPanel;
    RandomChartSource1: TRandomChartSource;
    StringGrid1: TStringGrid;
    procedure Chart1UserDrawnSeries1Draw(ACanvas: TCanvas; const ARect: TRect);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure fseAlphaChange(Sender: TObject);
  private
    Points: TDoublePointArray;
    Triangulation: TTriangulation;

  public

  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

procedure TForm1.Chart1UserDrawnSeries1Draw(ACanvas: TCanvas; const ARect: TRect);
const
  R = 3;
var
  i: Integer;
  P: TPoint;
  T: TDelaunayTriangle;
  TP: array[0..3] of TPoint;
  C: TPoint;
  sz: TSize;
  s: String;
begin
  ACanvas.Brush.Style := bsSolid;
  ACanvas.Brush.Color := clRed;
  for i := 0 to High(Points) do
  begin
    P := Chart1.GraphToImage(Points[i]);
    ACanvas.Ellipse(P.X-R, P.Y-R, P.X+R, P.Y+R);
  end;

  ACanvas.Pen.Color := clBlue;
  ACanvas.Brush.Style := bsClear;
  for i := 0 to High(Triangulation.Triangles) do
  begin
    T := Triangulation.Triangles[i];
    if T.VoronoiRadius < 0 then
      Continue;
    TP[0] := Chart1.GraphToImage(Points[T.VertexIndex[0]]);
    TP[1] := Chart1.GraphToImage(Points[T.VertexIndex[1]]);
    TP[2] := Chart1.GraphToImage(Points[T.VertexIndex[2]]);
    TP[3] := TP[0];
    ACanvas.Polyline(TP);
    C.X := (TP[0].X + TP[1].X + TP[2].X) div 3;
    C.Y := (TP[0].Y + TP[1].Y + TP[2].Y) div 3;
    s := IntToStr(i);
    sz := ACanvas.TextExtent(s);
    ACanvas.TextOut(C.X - sz.CX div 2, C.Y - sz.CY div 2, s);
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  i: Integer;
  t: TDelaunayTriangle;
begin
  SetLength(Points, RandomChartSource1.PointsNumber);
  for i := 0 to High(Points) do
  begin
    Points[i].X := RandomChartSource1.Item[i]^.X;
    Points[i].Y := RandomChartSource1.Item[i]^.XList[0];
  end;
  Chart1.LeftAxis.Range.Min := -1.2;
  Chart1.LeftAxis.Range.Max := 1.2;
  Chart1.LeftAxis.Range.UseMax := true;
  Chart1.LeftAxis.Range.UseMin := true;
  Chart1.BottomAxis.Range.Min := -1.2;
  Chart1.BottomAxis.Range.Max := 1.2;
  Chart1.BottomAxis.Range.UseMax := true;
  Chart1.BottomAxis.Range.UseMin := true;

  Triangulation := TTriangulation.Create(@Points[0], Length(Points));
  Triangulation.Execute;

  StringGrid1.RowCount := Length(Triangulation.Triangles) + 1;
  StringGrid1.ColCount := 4;
  StringGrid1.Cells[0, 0] := 'Triangle #';
  StringGrid1.Cells[1, 0] := 'Neighbors[0]';
  StringGrid1.Cells[2, 0] := 'Neighbors[1]';
  StringGrid1.Cells[3, 0] := 'Neighbors[2]';
  for i := 0 to High(Triangulation.Triangles) do
  begin
    t := Triangulation.Triangles[i];
    StringGrid1.Cells[0, i+1] := IntToStr(i);
    StringGrid1.Cells[1, i+1] := IntToStr(t.Neighbors[0]);
    StringGrid1.Cells[2, i+1] := IntToStr(t.Neighbors[1]);
    StringGrid1.Cells[3, i+1] := IntToStr(t.Neighbors[2]);
  end;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  Triangulation.Free;
end;

procedure TForm1.fseAlphaChange(Sender: TObject);
var
  i: Integer;
  R_crit: Double;
begin
  if fseAlpha.Value = 0 then  // normale Delaunay-Triangulation
    exit;

  Triangulation.Free;
  Triangulation := TTriangulation.Create(@Points[0], Length(Points));
  Triangulation.Execute;
  R_crit := 1.0 / fseAlpha.Value;
  for i := High(Triangulation.Triangles) downto 0 do
    if abs(Triangulation.Triangles[i].VoronoiRadius) > R_crit then
      Triangulation.Triangles[i].VoronoiRadius := -1;
  Chart1.Invalidate;
end;

end.

